/*
 * Copyright (c) 2000 Dave Jones <dave@powertweak.com>
 *
 * Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include "create-GUI.h"
#include "notebook.h"
#include "powertweak.h"
#include "config.h"
#include "intl.h"

#define SYSCTL_EXE "/sbin/sysctl -e -p /etc/sysctl.conf"
#define TMP_POSTFIX ".tmp"

extern GtkWidget *CreateMenus(GtkWidget *window);
extern void applyfunction (GtkWidget *widget, gpointer data);
extern void savefunction (GtkWidget *widget, gpointer data);
extern void quit_app (GtkWidget *widget, gpointer data);
extern void CreateTree(GtkWidget *scrolled_win);

/* getlocalesuffixed() func searchs and returns
   the filename with a locale suffix. If the
   config file is foobar.cfg and locale is ja_JP.eucJP,
   it will search:

   foobar.cfg.ja_JP.eucJP (Exactly match)
   foobar.cfg.ja_JP       (no codeset match)
   foobar.cfg.ja          (lang name match)

   and returns if it exists.

   Yukihiro Nakai <ynakai@redhat.com>
*/
char* getlocalesuffixed( char* filename ) {
  int fd=0;
  char* lang = NULL;
  char* langfull = NULL;
  char* langsmall = NULL;
  char* langmini = NULL;
  char* retval = NULL;
  char* tmpptr;

  if ( filename == NULL || *filename == '\0' ) return NULL;

  lang = setlocale(LC_ALL, "");
  if ( lang == NULL || *lang == '\0' ) return filename;
  if ( !strcmp(lang, "C") ) return filename;

  langfull = calloc(sizeof(char),strlen(filename)+strlen(lang)+2);
  if( langfull != NULL ) {
    strcpy(langfull, filename);
    strcat(langfull, ".");
    strcat(langfull, lang);
  }

  if( (tmpptr = strrchr(lang, '.')) != NULL ) {
    langsmall =
      calloc(sizeof(char), strlen(filename)+strlen(lang)-strlen(tmpptr)+2);
    if( langsmall != NULL ) {
      strcpy(langsmall, filename);
      strcat(langsmall, ".");
      strncat(langsmall, lang, strlen(lang)-strlen(tmpptr));
    }
  }

  if( (tmpptr = strchr(lang, '_')) != NULL ) {
    langmini =
      calloc(sizeof(char), strlen(filename)+strlen(lang)-strlen(tmpptr)+2);
    if( langmini != NULL ) {
      strcpy(langmini, filename);
      strcat(langmini, ".");
      strncat(langmini, lang, strlen(lang)-strlen(tmpptr));
    }
  }

  if ( langfull != NULL && (fd = open(langfull, O_RDONLY)) != -1 ) {
    close(fd);
    retval = strdup(langfull);
  } else if ( langsmall != NULL && (fd = open(langsmall, O_RDONLY)) != -1 ) {
    close(fd);
    retval = strdup(langsmall);
  } else if ( langmini != NULL && (fd = open(langmini, O_RDONLY)) != -1 ) {
    close(fd);
    retval = strdup(langmini);
  } else if ( (fd = open(filename, O_RDONLY)) != -1 ) {
    close(fd);
    retval = filename;
  }

  free(lang);
  free(langfull);
  free(langsmall);
  free(langmini);

  return retval;
}

void CreateGadgets(GtkWidget *window)
{
	GtkWidget *vbox;		/* Holds menubar & tree */
	GtkWidget *vbox2;		/* Holds notebook & buttons */
	GtkWidget *hbox;		/* Holds the apply/save/quit buttons */
	GtkWidget *Menubar;
	GtkWidget *scrolled_win;
	GtkWidget *lframe;
	GtkWidget *hpane;
	GtkText *TextPtr;
	GtkWidget *separator;
	GtkWidget *applybutton;
	GtkWidget *savebutton;
	GtkWidget *quitbutton;

	/* Create the pulldown menus */
	Menubar = CreateMenus (window);

	/* Horizontal pane to divide the GUI in two halves */
	hpane = gtk_hpaned_new();
	gtk_paned_handle_size (GTK_PANED (hpane), 10);
	gtk_paned_gutter_size (GTK_PANED (hpane), 10);

	/* Scrollable window for the tree in the LHS */
	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_set_usize (scrolled_win, WindowWidth/3, LeftWindowHeight);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_win),
									GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_show (scrolled_win);
	CreateTree (scrolled_win);

	/* Frame for the tree & scrollbar to go inside. */
	lframe = gtk_frame_new (_("System."));
	gtk_container_set_border_width (GTK_CONTAINER (lframe), 10);
	gtk_frame_set_shadow_type (GTK_FRAME (lframe), GTK_SHADOW_IN);
	gtk_frame_set_label_align (GTK_FRAME (lframe), 0, 0);
	gtk_widget_show (lframe);
	gtk_container_add (GTK_CONTAINER(lframe), scrolled_win);
	gtk_paned_add1 (GTK_PANED (hpane), lframe);

	/** Now the right hand side **/
	
	/* Create the notebook */
	notebook = (GTK_WIDGET(CreateNotebook(GTK_POS_TOP)));
	gtk_widget_set_usize (notebook, (WindowWidth/3)*2, RightWindowHeight);
	AddPage (GTK_NOTEBOOK (notebook), _("Sysctlconfig"));
	TextPtr = (GTK_TEXT(CreateText(g_strconcat(_("Sysctlconfig version: "), VERSION, NULL))));
	gtk_container_add (GTK_CONTAINER (current_pages[current_max_page-1]), GTK_WIDGET(TextPtr));
	gtk_widget_show (notebook);

	/* Pack & display the right hand side of the GUI */
	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox2), notebook, TRUE, TRUE, 0);
	gtk_paned_add2 (GTK_PANED (hpane), vbox2);
	gtk_widget_show (vbox2);
	gtk_widget_show (hpane);

	/* Now create the bottom part. */

	/* Separator bar */
	separator = gtk_hseparator_new();

	/* Create buttons. */
	applybutton = gtk_button_new_with_label (_("Activate saved configuration"));
	savebutton = gtk_button_new_with_label (_("Save"));
	quitbutton = gtk_button_new_with_label (_("Quit"));
	gtk_signal_connect (GTK_OBJECT (applybutton), "clicked", GTK_SIGNAL_FUNC (applyfunction), NULL);
	gtk_signal_connect (GTK_OBJECT (savebutton), "clicked", GTK_SIGNAL_FUNC (savefunction), NULL);
	gtk_signal_connect (GTK_OBJECT (quitbutton), "clicked", GTK_SIGNAL_FUNC (quit_app), NULL);
	/*	gtk_widget_set_sensitive (GTK_WIDGET (applybutton), FALSE ); */
	gtk_widget_show (applybutton);
	gtk_widget_show (savebutton);
	gtk_widget_show (quitbutton);

	/* Pack the buttons into the horizontal box */
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_container_border_width (GTK_CONTAINER (hbox), 20);
	gtk_box_pack_start (GTK_BOX (hbox), savebutton, TRUE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), applybutton, TRUE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), quitbutton, TRUE, FALSE, 0);
	gtk_widget_show (hbox);


	/* Pack & display the whole GUI. */
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), Menubar, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hpane, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), separator, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER(window), vbox);
	gtk_widget_show_all (window);
}

void applyfunction (GtkWidget __unused *widget, gpointer __unused gdata)
{
	char aerror_message[]="Error trying to run " SYSCTL_EXE;
	char applied_message[]="sysctl successfully applied";

	if (system(SYSCTL_EXE) != 0)
		show_popup ("Error!", aerror_message);
	else
		show_popup ("Done.", applied_message);
}
